/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.farm.model;

import com.vividsolutions.jts.geom.Envelope;
import com.vividsolutions.jts.geom.Geometry;
import com.vividsolutions.jts.geom.Point;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.Serializable;
import java.util.Date;
import nl.cloudfarming.client.field.model.Field;
import nl.cloudfarming.client.geoviewer.Geometrical;

/**
 * Model class for Crop fields
 * 
 * A CropField is part of a field where a certain crop is grown.
 * It is alwas part of a {@link CropProductionUnit} which contains
 * the crop that is meant to grow on this part of the field.
 * 
 * A CropField is temporal and is only valid between start and end date.
 * 
 * 
 * @author Gerben Feenstra
 */
public class CropField implements Serializable, Geometrical<Geometry> {

    private static final long serialVersionUID = 8763622679187376702L;
    public static final String EXT = "cfd";
    public static final String MIMETYPE = "application/x-agrosense-cropfield";
    private Geometry geometry;
    public static final String PROP_GEOMETRY = "geometry";
    private String name;
    public static final String PROP_NAME = "name";
    private double areaInHectares;
    public static final String PROP_AREA_IN_HECTARES = "areaInHectares";
    private Date startDate;
    public static final String PROP_START_DATE = "startDate";
    private Date endDate;
    public static final String PROP_END_DATE = "endDate";
    private final long id;
    private final long fieldId;
    private transient PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport(this);
    
    public CropField(Field field) {
        assert field != null:"Cannot create a crop field based on a null field";
        areaInHectares = field.getAreaInHectares();
        endDate = field.getEndDate();
        startDate = field.getStartDate();
        geometry = field.getGeometry();
        name = field.getName();
        fieldId = field.getId();
        id = CropFieldIdGenerator.Locator.getInstance().generateCropFieldId();
    }    

    @Override
    public Geometry getBoundingBox() {
        return (this.geometry!=null)?geometry.getBoundary():null;
    }    

    /**
     * Get the value of endDate
     *
     * @return the value of endDate
     */
    public Date getEndDate() {
        return endDate;
    }

    /**
     * Set the value of endDate
     *
     * @param endDate new value of endDate
     */
    public void setEndDate(Date endDate) {
        Date oldEndDate = this.endDate;
        this.endDate = endDate;
        propertyChangeSupport.firePropertyChange(PROP_END_DATE, oldEndDate, endDate);
    }

    /**
     * Get the value of startDate
     *
     * @return the value of startDate
     */
    public Date getStartDate() {
        return startDate;
    }

    /**
     * Set the value of startDate
     *
     * @param startDate new value of startDate
     */
    public void setStartDate(Date startDate) {
        Date oldStartDate = this.startDate;
        this.startDate = startDate;
        propertyChangeSupport.firePropertyChange(PROP_START_DATE, oldStartDate, startDate);
    }

    /**
     * Get the value of areaInHectares
     *
     * @return the value of areaInHectares
     */
    public double getAreaInHectares() {
        return areaInHectares;
    }

    /**
     * Set the value of areaInHectares
     *
     * @param areaInHectares new value of areaInHectares
     */
    public void setAreaInHectares(double areaInHectares) {
        double oldAreaInHectares = this.areaInHectares;
        this.areaInHectares = areaInHectares;
        propertyChangeSupport.firePropertyChange(PROP_AREA_IN_HECTARES, oldAreaInHectares, areaInHectares);
    }

    /**
     * Get the value of geometry
     *
     * @return the value of geometry
     */
    @Override
    public Geometry getGeometry() {
        return geometry;
    }

    /**
     * Set the value of geometry
     *
     * @param geometry new value of geometry
     */
    public void setGeometry(Geometry geometry) {
        Geometry oldGeometry = this.geometry;
        this.geometry = geometry;
        propertyChangeSupport.firePropertyChange(PROP_GEOMETRY, oldGeometry, geometry);
    }

    /**
     * Add PropertyChangeListener.
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        assert listener != null;
        propertyChangeSupport.addPropertyChangeListener(listener);
    }

    /**
     * Remove PropertyChangeListener.
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        propertyChangeSupport.removePropertyChangeListener(listener);
    }

    /**
     * Get the value of name
     *
     * @return the value of name
     */
    public String getName() {
        return name;
    }

    /**
     * Set the value of name
     *
     * @param name new value of name
     */
    public void setName(String name) {
        String oldName = this.name;
        this.name = name;
        propertyChangeSupport.firePropertyChange(PROP_NAME, oldName, name);
    }

    @Override
    public Point getCentroid() {
        return geometry.getCentroid();
    }

    @Override
    public Geometry getRenderObject(Envelope envelope) {
        return geometry;
    }

    @Override
    public String getTooltipText() {
        return getName();
    }

    @Override
    public String toString() {
        return this.name;
    }

    /**
     * Get the value of id
     *
     * @return the value of id
     */
    @Override
    public long getId() {
        return id;
    }

    /**
     * Get the value of fieldId
     *
     * @return the value of fieldId
     */
    public long getFieldId() {
        return fieldId;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final CropField other = (CropField) obj;
        if (this.id != other.id) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 67 * hash + (int) (this.id ^ (this.id >>> 32));
        return hash;
    }
    
    private void readObject(java.io.ObjectInputStream in)
            throws IOException, ClassNotFoundException {
        in.defaultReadObject();
        propertyChangeSupport = new PropertyChangeSupport(this);
    }

    @Override
    public String getIconPath() {
        return null;
    }

    @Override
    public String getIconLabel() {
        return null;
    }
}
