/**
 * Copyright (C) 2010-2012 Agrosense info@agrosense.eu
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.farm.model;

import java.util.Collection;
import java.util.logging.Logger;
import org.openide.util.Lookup;

/**
 * Generator for generating unique field id's
 * 
 * There should only be one implementation available in the application.
 * Use the Locator class to get an instance to guarantee only one instance
 * is loaded in the application
 * 
 * @author Timon Veenstra <monezz@gmail.com>
 */
public interface CropFieldIdGenerator {
    /**
     * Generates a unique field id for this installation.
     * Uniqueness across application installations or when
     * user directory is cleared is not part of the implementation contract.
     * @return unique field id
     */
    long generateCropFieldId();
    
    public static class Locator{
        private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.cropfield.model.CropFieldIdGenerator");
        
        /**
         * Failsafe convenience method to get a FieldIdGenerator implementation.
         * 
         * @return 
         */
        public static CropFieldIdGenerator getInstance(){
            Collection<? extends CropFieldIdGenerator> generators =  Lookup.getDefault().lookupAll(CropFieldIdGenerator.class);
            if (generators.size() > 1){
                throw new RuntimeException("FIXME: More then one unique id generators present in the application!");
            }
            if (generators.isEmpty()){
                LOGGER.warning("FIXME: No CropFieldIdGenerator implementation found, returning dummy implementation");
                return new CropFieldIdGenerator() {

                    @Override
                    public long generateCropFieldId() {
                        return 1l;
                    }
                };
            }            
            return generators.iterator().next();
        }
    }
}
